@@ -6,6 +6,7 @@ use parent 'App::cpm::Resolver::MetaDB';
 
 sub new {
     my $class = shift;
+    my $ctx = shift;
     my %package;
     for my $argv (@_) {
         my ($package, $fixed_version) = split /\@/, $argv;
@@ -17,7 +18,7 @@ sub new {
 }
 
 sub resolve {
-    my ($self, $argv) = @_;
+    my ($self, $ctx, $argv) = @_;
     my $fixed_version = $self->{_packages}{$argv->{package}};
     return { error => "not found" } if !$fixed_version;
     my $version_range = $argv->{version_range};
@@ -26,7 +27,7 @@ sub resolve {
     } else {
         $version_range = "== $fixed_version";
     }
-    $self->SUPER::resolve({ %$argv, version_range => $version_range });
+    $self->SUPER::resolve($ctx, { %$argv, version_range => $version_range });
 }
 
 1;
