@@ -6,6 +6,8 @@ use Config;
 use Cwd ();
 use Digest::MD5 ();
 use File::Spec;
+use File::Which ();
+use IPC::Run3 ();
 
 use Exporter 'import';
 
@@ -41,4 +43,19 @@ sub determine_home { # taken from Menlo
     File::Spec->catdir($homedir, ".perl-cpm");
 }
 
+my $gzip;
+sub gunzip {
+    my ($from, $to) = @_;
+    if (!$gzip) {
+        $gzip = File::Which::which('gzip');
+        die "need gzip command to decompress $from\n" if !$gzip;
+    }
+    my @cmd = ($gzip, "-dc", $from);
+    IPC::Run3::run3(\@cmd, undef, $to, \my $err);
+    return if $? == 0;
+    chomp $err;
+    $err ||= "exit status $?";
+    die "@cmd: $err\n";
+}
+
 1;
